@@ -30,6 +30,7 @@
 
 #include "simulation_info_json.h"
 #include "simulation_runtime.h"
+#include "options.h"
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
@@ -221,14 +222,18 @@ static const char* readEquation(const char *str,EQUATION_INFO *xml,int i)
   xml->id = i;
   str = skipFieldIfExist(str, "parent");
   str = skipFieldIfExist(str, "section");
-  if ((measure_time_flag & 1) && 0==strncmp(",\"tag\":\"container\"", str, 18)) {
+  if ((measure_time_flag & 1) && 0==strncmp(",\"tag\":\"system\"", str, 15)) {
     xml->profileBlockIndex = -1;
-    str += 18;
+    str += 15;
+  } else if ((measure_time_flag & 1) && 0==strncmp(",\"tag\":\"tornsystem\"", str, 19)) {
+    xml->profileBlockIndex = -1;
+    str += 19;
   } else {
     xml->profileBlockIndex = 0;
   }
   str = skipFieldIfExist(str, "tag");
   str = skipFieldIfExist(str, "display");
+  str = skipFieldIfExist(str, "unknowns");
   if (strncmp(",\"defines\":[", str, 12)) {
     xml->numVar = 0;
     xml->vars = 0;
@@ -367,15 +372,26 @@ static void readInfoJson(const char *str,MODEL_DATA_XML *xml)
 
 void modelInfoInit(MODEL_DATA_XML* xml)
 {
+#if !defined(OMC_NO_FILESYSTEM)
   omc_mmap_read mmap_reader = {0};
-  rt_tick(0);
+#endif
+  //rt_tick(0);
+#if !defined(OMC_NO_FILESYSTEM)
   if (!xml->infoXMLData) {
-    mmap_reader = omc_mmap_open_read(xml->fileName);
+    const char *filename;
+    if (omc_flag[FLAG_INPUT_PATH]) { /* read the input path from the command line (if any) */
+      if (0 > GC_asprintf(&filename, "%s/%s", omc_flagValue[FLAG_INPUT_PATH], xml->fileName)) {
+        throwStreamPrint(NULL, "simulation_info_json.c: Error: can not allocate memory.");
+      }
+      mmap_reader = omc_mmap_open_read(filename);
+    } else {
+      mmap_reader = omc_mmap_open_read(xml->fileName);
+    }
     xml->infoXMLData = mmap_reader.data;
     xml->modelInfoXmlLength = mmap_reader.size;
     // fprintf(stderr, "Loaded the JSON (%ld kB)...\n", (long) (s.st_size+1023)/1024);
   }
-
+#endif
   xml->functionNames = (FUNCTION_INFO*) calloc(xml->nFunctions, sizeof(FUNCTION_INFO));
   xml->equationInfo = (EQUATION_INFO*) calloc(1+xml->nEquations, sizeof(EQUATION_INFO));
   xml->equationInfo[0].id = 0;
@@ -388,7 +404,9 @@ void modelInfoInit(MODEL_DATA_XML* xml)
   // fprintf(stderr, "Parse the JSON %ld...\n", (long) xml->infoXMLData);
   readInfoJson(xml->infoXMLData, xml);
   // fprintf(stderr, "Parsed the JSON in %fms...\n", rt_tock(0) * 1000.0);
+#if !defined(OMC_NO_FILESYSTEM)
   omc_mmap_close_read(mmap_reader);
+#endif
 }
 
 FUNCTION_INFO modelInfoGetFunction(MODEL_DATA_XML* xml, size_t ix)
