@@ -1,33 +1,32 @@
 /*
-* This file is part of OpenModelica.
-*
-* Copyright (c) 1998-CurrentYear, Linköping University,
-* Department of Computer and Information Science,
-* SE-58183 Linköping, Sweden.
-*
-* All rights reserved.
-*
-* THIS PROGRAM IS PROVIDED UNDER THE TERMS OF GPL VERSION 3
-* AND THIS OSMC PUBLIC LICENSE (OSMC-PL).
-* ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES RECIPIENT'S
-* ACCEPTANCE OF THE OSMC PUBLIC LICENSE.
-*
-* The OpenModelica software and the Open Source Modelica
-* Consortium (OSMC) Public License (OSMC-PL) are obtained
-* from Linköping University, either from the above address,
-* from the URLs: http://www.ida.liu.se/projects/OpenModelica or
-* http://www.openmodelica.org, and in the OpenModelica distribution.
-* GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
-*
-* This program is distributed WITHOUT ANY WARRANTY; without
-* even the implied warranty of  MERCHANTABILITY or FITNESS
-* FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
-* IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS
-* OF OSMC-PL.
-*
-* See the full OSMC Public License conditions for more details.
-*
-*/
+ * This file is part of OpenModelica.
+ *
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
+ * c/o Linköpings universitet, Department of Computer and Information Science,
+ * SE-58183 Linköping, Sweden.
+ *
+ * All rights reserved.
+ *
+ * THIS PROGRAM IS PROVIDED UNDER THE TERMS OF THE BSD NEW LICENSE OR THE
+ * GPL VERSION 3 LICENSE OR THE OSMC PUBLIC LICENSE (OSMC-PL) VERSION 1.2.
+ * ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES
+ * RECIPIENT'S ACCEPTANCE OF THE OSMC PUBLIC LICENSE OR THE GPL VERSION 3,
+ * ACCORDING TO RECIPIENTS CHOICE.
+ *
+ * The OpenModelica software and the OSMC (Open Source Modelica Consortium)
+ * Public License (OSMC-PL) are obtained from OSMC, either from the above
+ * address, from the URLs: http://www.openmodelica.org or
+ * http://www.ida.liu.se/projects/OpenModelica, and in the OpenModelica
+ * distribution. GNU version 3 is obtained from:
+ * http://www.gnu.org/copyleft/gpl.html. The New BSD License is obtained from:
+ * http://www.opensource.org/licenses/BSD-3-Clause.
+ *
+ * This program is distributed WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, EXCEPT AS
+ * EXPRESSLY SET FORTH IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE
+ * CONDITIONS OF OSMC-PL.
+ *
+ */
 
 /*! \file simulation_data.h
  * Description: This is the C header file to provide all information
@@ -153,7 +152,6 @@ typedef struct ANALYTIC_JACOBIAN
   modelica_real* seedVars;
   modelica_real* tmpVars;
   modelica_real* resultVars;
-  modelica_real* jacobian;
 }ANALYTIC_JACOBIAN;
 
 /* EXTERNAL_INPUT
@@ -223,6 +221,7 @@ typedef struct STATIC_REAL_DATA
   VAR_INFO info;
   REAL_ATTRIBUTE attribute;
   modelica_boolean filterOutput;       /* true if this variable should be filtered */
+  modelica_boolean time_unvarying;     /* true if the value is only computed once during initialization */
 }STATIC_REAL_DATA;
 
 typedef struct STATIC_INTEGER_DATA
@@ -230,6 +229,7 @@ typedef struct STATIC_INTEGER_DATA
   VAR_INFO info;
   INTEGER_ATTRIBUTE attribute;
   modelica_boolean filterOutput;       /* true if this variable should be filtered */
+  modelica_boolean time_unvarying;     /* true if the value is only computed once during initialization */
 }STATIC_INTEGER_DATA;
 
 typedef struct STATIC_BOOLEAN_DATA
@@ -237,6 +237,7 @@ typedef struct STATIC_BOOLEAN_DATA
   VAR_INFO info;
   BOOLEAN_ATTRIBUTE attribute;
   modelica_boolean filterOutput;       /* true if this variable should be filtered */
+  modelica_boolean time_unvarying;     /* true if the value is only computed once during initialization */
 }STATIC_BOOLEAN_DATA;
 
 typedef struct STATIC_STRING_DATA
@@ -244,6 +245,7 @@ typedef struct STATIC_STRING_DATA
   VAR_INFO info;
   STRING_ATTRIBUTE attribute;
   modelica_boolean filterOutput;       /* true if this variable should be filtered */
+  modelica_boolean time_unvarying;     /* true if the value is only computed once during initialization */
 }STATIC_STRING_DATA;
 
 #if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
@@ -253,6 +255,7 @@ typedef struct NONLINEAR_SYSTEM_DATA
   modelica_integer equationIndex;      /* index for EQUATION_INFO */
 
   modelica_boolean homotopySupport;    /* 1 if homotopy is available, 0 otherwise */
+  modelica_boolean initHomotopy;       /* 1 if the homotopy solver should be used to solve the initial system, 0 otherwise */
   modelica_boolean mixedSystem;        /* 1 if the system contains discrete variables, 0 otherwise */
   /* attributes of iteration variables */
   modelica_real *min;
@@ -265,8 +268,8 @@ typedef struct NONLINEAR_SYSTEM_DATA
    *
    * if analyticalJacobianColumn == NULL no analyticalJacobian is available
    */
-  int (*analyticalJacobianColumn)(void*, threadData_t*);
-  int (*initialAnalyticalJacobian)(void*, threadData_t*);
+  int (*analyticalJacobianColumn)(void*, threadData_t*, ANALYTIC_JACOBIAN*, ANALYTIC_JACOBIAN* parentJacobian);
+  int (*initialAnalyticalJacobian)(void*, threadData_t*, ANALYTIC_JACOBIAN*);
   modelica_integer jacobianIndex;
 
   SPARSE_PATTERN sparsePattern;        /* sparse pattern if no jacobian is available */
@@ -317,8 +320,8 @@ typedef struct LINEAR_SYSTEM_DATA
   void (*setAElement)(int row, int col, double value, int nth, void *data, threadData_t *threadData);
   void (*setBElement)(int row, double value, void *data, threadData_t *threadData);
 
-  int (*analyticalJacobianColumn)(void*, threadData_t*);
-  int (*initialAnalyticalJacobian)(void*, threadData_t*);
+  int (*analyticalJacobianColumn)(void*, threadData_t*, ANALYTIC_JACOBIAN*, ANALYTIC_JACOBIAN* parentJacobian);
+  int (*initialAnalyticalJacobian)(void*, threadData_t*, ANALYTIC_JACOBIAN*);
   modelica_integer jacobianIndex;
 
   void (*residualFunc)(void**, const double*, double*, const int*);
@@ -335,7 +338,7 @@ typedef struct LINEAR_SYSTEM_DATA
   modelica_integer size;
   modelica_integer equationIndex;       /* index for EQUATION_INFO */
 
-  void *solverData;
+  void *solverData[2]; /* [1] is the totalPivot solver; [0] holds other solvers ; both are used for the default solver */
   modelica_real *x;                     /* solution vector x */
   modelica_real *A;                     /* matrix A */
   modelica_real *b;                     /* vector b */
@@ -345,6 +348,7 @@ typedef struct LINEAR_SYSTEM_DATA
   modelica_boolean solved;              /* 1: solved in current step - else not */
   modelica_boolean failed;              /* 1: failed while last try with lapack - else not */
   modelica_boolean useSparseSolver;     /* 1: use sparse solver, - else any solver */
+  ANALYTIC_JACOBIAN* parentJacobian; 	/* if != NULL then it's the parent jacobian matrix */
 
   /* statistics */
   unsigned long numberOfCall;           /* number of solving calls of this system */
@@ -401,8 +405,8 @@ typedef struct STATE_SET_DATA
    *
    * if analyticalJacobianColumn == NULL no analyticalJacobian is available
    */
-  int (*analyticalJacobianColumn)(void*, threadData_t*);
-  int (*initialAnalyticalJacobian)(void*, threadData_t*);
+  int (*analyticalJacobianColumn)(void*, threadData_t*, ANALYTIC_JACOBIAN*, ANALYTIC_JACOBIAN* parentJacobian);
+  int (*initialAnalyticalJacobian)(void*, threadData_t*, ANALYTIC_JACOBIAN*);
   modelica_integer jacobianIndex;
 }STATE_SET_DATA;
 #else
@@ -417,23 +421,23 @@ typedef struct DAEMODE_DATA
   /* number of algebraic variables */
   long nAlgebraicDAEVars;
 
+  /* number of algebraic variables */
+  long nAuxiliaryVars;
+
   /* workspace for the residual variables */
   modelica_real* residualVars;
 
+  /* workspace for the auxiliary variables */
+  modelica_real* auxiliaryVars;
+
   /* daeMode sparse pattern */
   SPARSE_PATTERN* sparsePattern;
 
   /* function to evaluate dynamic equations for DAE solver*/
-  int (*evaluateDAEResiduals)(struct DATA*, threadData_t*);
+  int (*evaluateDAEResiduals)(struct DATA*, threadData_t*, int);
 
-  /* function to set algebraic DAE Variable form solver*/
-  int (*setAlgebraicDAEVars)(struct DATA*, threadData_t*, double*);
-
-  /* function to get algebraic DAE Variable form solver*/
-  int (*getAlgebraicDAEVars)(struct DATA*, threadData_t*, double*);
-
-  /* function to get algebraic DAE nominal values*/
-  int (*getAlgebraicDAEVarNominals)(struct DATA*, threadData_t*, double*);
+  /* index of the algebraic DAE variable in original order */
+  int *algIndexes;
 
 } DAEMODE_DATA;
 
@@ -499,6 +503,7 @@ typedef struct MODEL_DATA
   const char* modelDir;
   const char* modelGUID;
   const char* initXMLData;
+  char* resourcesDir;
 
   long nSamples;                       /* number of different sample-calls */
   SAMPLE_INFO* samplesInfo;            /* array containing each sample-call */
@@ -559,6 +564,7 @@ enum EVAL_CONTEXT
   CONTEXT_ALGEBRAIC,
   CONTEXT_EVENTS,
   CONTEXT_JACOBIAN,
+  CONTEXT_SYM_JACOBIAN,
 
   CONTEXT_MAX
 };
@@ -590,7 +596,7 @@ typedef struct SIMULATION_INFO
   int jacobianEvals;                   /* number of different columns to evaluate functionODE */
   int currentJacobianEval;             /* current column to evaluate functionODE for Jacobian*/
 
-  int homotopyUsed;                    /* =1 the initialization problem was solved with a homotopy method, =0 otherwise */
+  int homotopySteps;                   /* the number of homotopy lambda steps during initialization, =0 no homotopy was used */
   double lambda;                       /* homotopy parameter E [0, 1.0] */
 
   /* indicators for simulations state */
