@@ -38,6 +38,11 @@
 extern "C" {
 #endif
 
+/* make sure all compiler use the same alignment policies for structures */
+#if defined _MSC_VER || defined __GNUC__
+#pragma pack(push,8)
+#endif
+
 // categories of logging supported by model.
 // Value is the index in logCategories of a ModelInstance.
 #define LOG_EVENTS                      0
@@ -76,7 +81,7 @@ typedef struct {
   fmi2EventInfo eventInfo;
 
   DATA* fmuData;
-  threadData_t *threadData;
+  threadData_t *threadData, *threadDataParent;
   fmi2Boolean toleranceDefined;
   fmi2Real tolerance;
   fmi2Real startTime;
@@ -84,11 +89,15 @@ typedef struct {
   fmi2Real stopTime;
 
   int _need_update;
-#ifdef FMU_EXPERIMENTAL
   int _has_jacobian;
-#endif
+  ANALYTIC_JACOBIAN* fmiDerJac;
 } ModelInstance;
 
+/* reset alignment policy to the one set before reading this file */
+#if defined _MSC_VER || defined __GNUC__
+#pragma pack(pop)
+#endif
+
 #ifdef __cplusplus
 }
 #endif
