@@ -43,9 +43,9 @@
 #include "sundials/sundials_dense.h" /* definitions DlsMat DENSE_ELEM */
 #include "sundials/sundials_types.h" /* definition of type realtype */
 
-int oms3::cvode_rhs(realtype t, N_Vector y, N_Vector ydot, void* user_data)
+int oms::cvode_rhs(realtype t, N_Vector y, N_Vector ydot, void* user_data)
 {
-  //std::cout << "\n[oms2::cvode_rhs] t=" << t << std::endl;
+  //std::cout << "\n[oms::cvode_rhs] t=" << t << std::endl;
   SystemSC* system = (SystemSC*)user_data;
   oms_status_enu_t status;
 
@@ -62,7 +62,7 @@ int oms3::cvode_rhs(realtype t, N_Vector y, N_Vector ydot, void* user_data)
     status = system->fmus[i]->setContinuousStates(system->states[i]);
     if (oms_status_ok != status) return status;
   }
-  //std::cout << "[oms3::cvode_rhs] y" << std::endl;
+  //std::cout << "[oms::cvode_rhs] y" << std::endl;
   //N_VPrint_Serial(y);
 
   system->updateInputs(system->outputsGraph);
@@ -84,16 +84,16 @@ int oms3::cvode_rhs(realtype t, N_Vector y, N_Vector ydot, void* user_data)
   return 0;
 }
 
-oms3::SystemSC::SystemSC(const ComRef& cref, Model* parentModel, System* parentSystem)
-  : oms3::System(cref, oms_system_sc, parentModel, parentSystem)
+oms::SystemSC::SystemSC(const ComRef& cref, Model* parentModel, System* parentSystem)
+  : oms::System(cref, oms_system_sc, parentModel, parentSystem, oms_solver_sc_cvode)
 {
 }
 
-oms3::SystemSC::~SystemSC()
+oms::SystemSC::~SystemSC()
 {
 }
 
-oms3::System* oms3::SystemSC::NewSystem(const oms3::ComRef& cref, oms3::Model* parentModel, oms3::System* parentSystem)
+oms::System* oms::SystemSC::NewSystem(const oms::ComRef& cref, oms::Model* parentModel, oms::System* parentSystem)
 {
   if (!cref.isValidIdent())
   {
@@ -111,34 +111,34 @@ oms3::System* oms3::SystemSC::NewSystem(const oms3::ComRef& cref, oms3::Model* p
   return system;
 }
 
-std::string oms3::SystemSC::getSolverName() const
+std::string oms::SystemSC::getSolverName() const
 {
   switch (solverMethod)
   {
-    case oms_solver_explicit_euler:
+    case oms_solver_sc_explicit_euler:
       return std::string("euler");
-    case oms_solver_cvode:
+    case oms_solver_sc_cvode:
       return std::string("cvode");
   }
 
   return std::string("unknown");
 }
 
-oms_status_enu_t oms3::SystemSC::setSolverMethod(std::string solver)
+oms_status_enu_t oms::SystemSC::setSolverMethod(std::string solver)
 {
   if (std::string("euler") == solver)
-    solverMethod = oms_solver_explicit_euler;
+    solverMethod = oms_solver_sc_explicit_euler;
   else if (std::string("cvode") == solver)
-    solverMethod = oms_solver_cvode;
+    solverMethod = oms_solver_sc_cvode;
   else
     return oms_status_error;
 
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::exportToSSD_SimulationInformation(pugi::xml_node& node) const
+oms_status_enu_t oms::SystemSC::exportToSSD_SimulationInformation(pugi::xml_node& node) const
 {
-  pugi::xml_node node_simulation_information = node.append_child(oms2::ssd::ssd_simulation_information);
+  pugi::xml_node node_simulation_information = node.append_child(oms::ssd::ssd_simulation_information);
 
   pugi::xml_node node_solver = node_simulation_information.append_child("VariableStepSolver");
   node_solver.append_attribute("description") = getSolverName().c_str();
@@ -151,7 +151,7 @@ oms_status_enu_t oms3::SystemSC::exportToSSD_SimulationInformation(pugi::xml_nod
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::importFromSSD_SimulationInformation(const pugi::xml_node& node)
+oms_status_enu_t oms::SystemSC::importFromSSD_SimulationInformation(const pugi::xml_node& node)
 {
   std::string solverName = node.child("VariableStepSolver").attribute("description").as_string();
   if (oms_status_ok != setSolverMethod(solverName))
@@ -164,8 +164,10 @@ oms_status_enu_t oms3::SystemSC::importFromSSD_SimulationInformation(const pugi:
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::instantiate()
+oms_status_enu_t oms::SystemSC::instantiate()
 {
+  time = getModel()->getStartTime();
+
   // there shouldn't be any substem
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->instantiate())
@@ -197,13 +199,13 @@ oms_status_enu_t oms3::SystemSC::instantiate()
 
   if (n_states == 0)
   {
-    solverMethod = oms_solver_explicit_euler;
+    solverMethod = oms_solver_sc_explicit_euler;
     logInfo("model doesn't contain any continuous state");
   }
 
-  if (oms_solver_explicit_euler == solverMethod)
+  if (oms_solver_sc_explicit_euler == solverMethod)
     ;
-  else if (oms_solver_cvode == solverMethod)
+  else if (oms_solver_sc_cvode == solverMethod)
     solverData.cvode.mem = NULL;
   else
     return logError_InternalError;
@@ -211,12 +213,16 @@ oms_status_enu_t oms3::SystemSC::instantiate()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::initialize()
+oms_status_enu_t oms::SystemSC::initialize()
 {
   clock.reset();
   CallClock callClock(clock);
 
-  time = getModel()->getStartTime();
+  if (oms_status_ok != updateDependencyGraphs())
+    return oms_status_error;
+
+  if (oms_status_ok != updateInputs(initialUnknownsGraph))
+    return oms_status_error;
 
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->initialize())
@@ -246,7 +252,7 @@ oms_status_enu_t oms3::SystemSC::initialize()
     }
   }
 
-  if (oms_solver_cvode == solverMethod)
+  if (oms_solver_sc_cvode == solverMethod)
   {
     size_t n_states = 0;
     for (int i=0; i < fmus.size(); ++i)
@@ -314,16 +320,10 @@ oms_status_enu_t oms3::SystemSC::initialize()
     if (flag < 0) logError("SUNDIALS_ERROR: CVodeSetMaxNumSteps() failed with flag = " + std::to_string(flag));
   }
 
-  if (oms_status_ok != updateDependencyGraphs())
-    return oms_status_error;
-
-  if (oms_status_ok != updateInputs(initialUnknownsGraph))
-    return oms_status_error;
-
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::terminate()
+oms_status_enu_t oms::SystemSC::terminate()
 {
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->terminate())
@@ -333,7 +333,7 @@ oms_status_enu_t oms3::SystemSC::terminate()
     if (oms_status_ok != component.second->terminate())
       return oms_status_error;
 
-  if (oms_solver_cvode == solverMethod && solverData.cvode.mem)
+  if (oms_solver_sc_cvode == solverMethod && solverData.cvode.mem)
   {
     long int nst, nfe, nsetups, nni, ncfn, netf;
     int flag;
@@ -385,7 +385,7 @@ oms_status_enu_t oms3::SystemSC::terminate()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::reset()
+oms_status_enu_t oms::SystemSC::reset()
 {
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->reset())
@@ -395,7 +395,9 @@ oms_status_enu_t oms3::SystemSC::reset()
     if (oms_status_ok != component.second->reset())
       return oms_status_error;
 
-  if (oms_solver_cvode == solverMethod && solverData.cvode.mem)
+  time = getModel()->getStartTime();
+
+  if (oms_solver_sc_cvode == solverMethod && solverData.cvode.mem)
   {
     long int nst, nfe, nsetups, nni, ncfn, netf;
     int flag;
@@ -427,7 +429,7 @@ oms_status_enu_t oms3::SystemSC::reset()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::stepUntil(double stopTime, void (*cb)(const char* ident, double time, oms_status_enu_t status))
+oms_status_enu_t oms::SystemSC::stepUntil(double stopTime, void (*cb)(const char* ident, double time, oms_status_enu_t status))
 {
   CallClock callClock(clock);
 
@@ -498,7 +500,7 @@ oms_status_enu_t oms3::SystemSC::stepUntil(double stopTime, void (*cb)(const cha
         status = fmus[i]->getEventindicators(event_indicators[i]);
         if (oms_status_ok != status) return status;
 
-        if (oms_solver_cvode == solverMethod)
+        if (oms_solver_sc_cvode == solverMethod)
         {
           size_t offset=0;
           for (size_t k=0; k < i; ++k)
@@ -525,7 +527,7 @@ oms_status_enu_t oms3::SystemSC::stepUntil(double stopTime, void (*cb)(const cha
     }
 
     // integrate using specified solver
-    if (oms_solver_explicit_euler == solverMethod)
+    if (oms_solver_sc_explicit_euler == solverMethod)
     {
       for (int i=0; i < fmus.size(); ++i)
       {
@@ -547,7 +549,7 @@ oms_status_enu_t oms3::SystemSC::stepUntil(double stopTime, void (*cb)(const cha
       // set time
       time = tnext;
     }
-    else if (oms_solver_cvode == solverMethod)
+    else if (oms_solver_sc_cvode == solverMethod)
     {
       double cvode_time = tlast;
       int flag = CVode(solverData.cvode.mem, tnext, solverData.cvode.y, &cvode_time, CV_NORMAL);
@@ -606,7 +608,7 @@ oms_status_enu_t oms3::SystemSC::stepUntil(double stopTime, void (*cb)(const cha
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::updateInputs(DirectedGraph& graph)
+oms_status_enu_t oms::SystemSC::updateInputs(DirectedGraph& graph)
 {
   CallClock callClock(clock);
 
@@ -648,7 +650,7 @@ oms_status_enu_t oms3::SystemSC::updateInputs(DirectedGraph& graph)
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemSC::solveAlgLoop(DirectedGraph& graph, const std::vector< std::pair<int, int> >& SCC)
+oms_status_enu_t oms::SystemSC::solveAlgLoop(DirectedGraph& graph, const std::vector< std::pair<int, int> >& SCC)
 {
   CallClock callClock(clock);
 
